//      www.kendziorra.nl
// (Philips) AirFryer project v2
// nov 2019, free to edit and use
/////////////////////////////////
//          CAUTION            //
//  Use this AirFryer project  //
//    only at you own risk!    //
//    I will not accept any    //
//          liability!         //
/////////////////////////////////
// use a ESP32 DEV-KIT module  //
/////////////////////////////////

#ifndef _AIRFRYER_H_
  #define _AIRFRYER_H_
  //===============================
  #include <WebSocketsServer.h>  // https://github.com/Links2004/arduinoWebSockets
  //==============================  included - http://kendziorra.nl
  #include <sTimers.h>           // simple libraries www.kendziorra.nl
  #include <sEspAdc.h>
  #include <sThermistor.h>
  //==============================   
  #include "EditMe.h"            // EDIT THIS 
  
  enum {
    rFan, rHeat,              // relays Fan, Heat
    nFryer = 0, nProbe,       // NTC 104CT, 204CT
    tRed = 0, tGreen,         // TOUCH, red/green pause  (buttons)
    tGo = 0, tShake, tPause,  // SECONDS: to go, to shake, max pause 
    sStage1_1 = 0,            // STAGES init, msg from enum-bStages-fieds _1: temp, _2: minutes to go, _3: minutes to shake
    sStage2_1, sStage2_2, sStage2_3,
    sStage3_1, sStage3_2, sStage3_3,
    sStage4_1, sStage4_2, sStage4_3,
               sStage5_2, sStage5_3
  };
  const uint8_t maxTmp = 200;               // max. fryer temp
  char msgBuf[60],                          // WebSocket inbuffer
       msgFields[13][16];                   // fryer msg-init-fields from msgBuf
  uint8_t stageNr,                          // STAGE NUMBER: holt the active stage number, 1(1), 2(1,2,3), 3(1,2,3), 4(1,2,3) or 5(2,3)
          overshoot = 4,                    // offset before the requested temperature
          minTmp = 30,                      // minimum heating ON temperature
          targetTmp = 0;                    // stage targed temp
  uint16_t temperature[] = {0, 0},          // temp from sensors NTC 104CT, 204CT
           remainingTime[] = {0, 0, 0};     // stages heat, shake and pause count down times
  int16_t stgValues[12];                    // STAGES init, msg from enum-sStages-fields
  bool fanOn = false,                       // fan off then heat off
       heatOn = false,                      // heat off
       fryerStart = false,                  // fryer start/stop
       fryerPause = false,                  // on RED-touch pause, heat and fan OFF
       useProbe = false;                    // using meat probe No/YES (NTC 204CT)
  volatile uint8_t touchDetect[] = {0, 0};  // TOUCH, red/green indication

  WebSocketsServer webSocket = WebSocketsServer(wsPort, wsURI, wsProtocol);
  sTimers tDebounce(2000, false);  // debounce timer: 2 sec., false: timer OFF
  sTimers tTimer(2000, true);      // stages timer: 2 sec. count down, true: timer ON
  sThermistors sFryer;
  sThermistors sProbe;

  void doCmd();             // do Start/Stop command from WebSocket
  void doStop();            // from WebSocket or if ready
  void setStage(uint8_t stage);  // set the individuals stage values
  void doStages();          // perform the individuals stages
  void setFan(bool on);     // if pause then OFF (RED touch)
  void setHeat(bool on);    // only if fan is ON
  void doRed();             // pause touch
  void doGreen();           // get on (after pause)
  void doMsg();             // message to WebSocket
  void getTemp(uint8_t n);  // get temp and resistance from NTC 104CT, 204CT
  bool msgParse();          // extract WebSocket message to fields
  void redInterrupt();      // pause touch
  void greenInterrupt();    // get on (after pause)
  void webSocketEvent(uint8_t, WStype_t, uint8_t *, size_t);  // WebSocketServer

  void doCmd() {
    if(!msgParse()) return;  // extract msg to char fields, if no fields return
    if (strcmp(msgFields[0], "stop") == 0) {doStop(); return;}  // stop cmd from WebSocket
    if (strcmp(msgFields[0], "start") == 0) {  // start cmd from WebSocket, init the stages
      int16_t s1    =  atoi(msgFields[sStage1_1 + 1]),  // all stages: char to integer
              s2[3] = {atoi(msgFields[sStage2_1 + 1]), atoi(msgFields[sStage2_2 + 1]), atoi(msgFields[sStage2_3 + 1])},
              s3[3] = {atoi(msgFields[sStage3_1 + 1]), atoi(msgFields[sStage3_2 + 1]), atoi(msgFields[sStage3_3 + 1])},
              s4[3] = {atoi(msgFields[sStage4_1 + 1]), atoi(msgFields[sStage4_2 + 1]), atoi(msgFields[sStage4_3 + 1])},
              s5[2] = {atoi(msgFields[sStage5_2 + 1]), atoi(msgFields[sStage5_3 + 1])};
      for (uint8_t i = 0; i <= sStage5_3; i++) stgValues[i] = 0;  // all stage values to int 0
      stgValues[sStage1_1] = s1 >= minTmp ? s1 : 0;  // stage 1, preheat
      if (s2[0] >= minTmp) {  // stage 2
        stgValues[sStage2_1] = s2[0];  // temp
        stgValues[sStage2_2] = s2[1] > 0 ? s2[1] * 60 : 0;  // time to go (seconds)
        stgValues[sStage2_3] = s2[2] > 0 ? s2[2] * 60 : 0;  // time to shake (seconds)
      }
      if (s3[0] >= minTmp) {  // stage 3, normal or with meat probe
        stgValues[sStage3_1] = s3[0];
        stgValues[sStage3_2] = s3[1] > 0 ? s3[1] * 60 : s3[1];  // if neg value then temp meat probe
        stgValues[sStage3_3] = s3[2] > 0 ? s3[2] * 60 : 0;
      }
      if (s4[0] >= minTmp) {  // stage 4
        stgValues[sStage4_1] = s4[0];
        stgValues[sStage4_2] = s4[1] > 0 ? s4[1] * 60 : 0;
        stgValues[sStage4_3] = s4[2] > 0 ? s4[2] * 60 : 0; 
      }
      if (s5[0] >= 0) {  // stage 5, meat rest time
        stgValues[sStage5_2] = s5[0] * 60;         // time to go (seconds)
        stgValues[sStage5_3] = s5[1] > 0 ? 1 : 0;  // fan ON/OFF
      }
      fryerStart = true;
      setStage(sStage1_1);
    }
  }

  void doStop() { 
    setFan(false);
    setHeat(false);
    fryerStart = false;
    fryerPause = false;
    useProbe = false;
    webSocket.broadcastTXT("|STOP|");
  }

  void setStage(uint8_t stage) {
    useProbe = false;
    targetTmp = 0;
    stageNr = stage;
    remainingTime[tShake] = 0;
    switch (stageNr) {
      case sStage1_1: 
        targetTmp = stgValues[sStage1_1] > maxTmp ? maxTmp : stgValues[sStage1_1];
        remainingTime[tGo] = 0;
        break;
      case sStage2_1: 
        targetTmp = stgValues[sStage2_1] > maxTmp ? maxTmp : stgValues[sStage2_1];
        remainingTime[tGo] = stgValues[sStage2_2];
        remainingTime[tShake] = stgValues[sStage2_3];
        break;
      case sStage3_1: 
        targetTmp = stgValues[sStage3_1] > maxTmp ? maxTmp : stgValues[sStage3_1];
        if (stgValues[sStage3_2] <= -20) {
          stgValues[sStage3_2] = abs(stgValues[sStage3_2]);
          if (stgValues[sStage3_2] > 90) stgValues[sStage3_2] = 90;
          useProbe = true;  // meat probe
          remainingTime[tGo] = 0;
        }
        else remainingTime[tGo] = stgValues[sStage3_2];
        remainingTime[tShake] = stgValues[sStage3_3];
        break;
      case sStage4_1:
        targetTmp = stgValues[sStage4_1] > maxTmp ? maxTmp : stgValues[sStage4_1];
        remainingTime[tGo] = stgValues[sStage4_2];
        remainingTime[tShake] = stgValues[sStage4_3];
        break;
      case sStage5_2:
        remainingTime[tGo] = stgValues[sStage5_2];
        setFan(stgValues[sStage5_3] ? true : false);
        break;
    }
    if (stageNr == sStage1_1) strcpy(msgBuf, "|start|");
    else strcpy(msgBuf, "|next|");
    webSocket.broadcastTXT(msgBuf);
    doMsg();
  }

  void doStages() {
    if (useProbe) getTemp(nProbe);  // probe and fryer
    else getTemp(nFryer);  // only fryer
    if (temperature[nFryer] > (maxTmp + 15)) {  // panic?
      doStop();
      webSocket.broadcastTXT("|OVERHEATED|");
      return;
    }
    if (fryerPause) {
      if (remainingTime[tPause] >= 2) remainingTime[tPause] -= 2;
      else doStop(); // stop fryer after 5 minutes pause
    }
    else {
      setFan(true);
      if (targetTmp >= minTmp) if (temperature[nFryer] < (targetTmp - overshoot)) setHeat(true);
      else setHeat(false);
      if (remainingTime[tGo] >= 2) remainingTime[tGo] -= 2;
      if (remainingTime[tShake] >= 2) remainingTime[tShake] -= 2;
      doMsg();
      switch (stageNr) {
        case sStage1_1:
          if (targetTmp == 0) setStage(sStage2_1);  // skip this stage
          else if (temperature[nFryer] >= targetTmp) webSocket.broadcastTXT("|insert|"); // Stage2 start after touch GREEN
          break;
        case sStage2_1:
          if (targetTmp == 0 || remainingTime[tGo] == 0) setStage(sStage3_1);
          else if (remainingTime[tShake] == 0) {
            remainingTime[tShake] = stgValues[sStage2_3];
            if (remainingTime[tShake]) webSocket.broadcastTXT("|shake|");
          }
          break;
        case sStage3_1:
          if (useProbe) {
            if (temperature[nProbe] >= stgValues[sStage3_2]) setStage(sStage4_1);
          }
          else {
            if (targetTmp == 0 || remainingTime[tGo] == 0) setStage(sStage4_1);      
            else if (remainingTime[tShake] == 0) {
              remainingTime[tShake] = stgValues[sStage3_3];
              if (remainingTime[tShake]) webSocket.broadcastTXT("|shake|");
            }
          }
          break;
        case sStage4_1:
          if (targetTmp == 0 || remainingTime[tGo] == 0) setStage(sStage5_2);
          else if (remainingTime[tShake] == 0) {
            remainingTime[tShake] = stgValues[sStage4_3];
            if (remainingTime[tShake]) webSocket.broadcastTXT("|shake|");
          }
          break;
        case sStage5_2:
          if (remainingTime[tGo] == 0) {
            doStop();
            webSocket.broadcastTXT("|ready|");
          }
          break;
      }
    }
  }

  void setFan(bool fan) {
    fanOn = fan;
    digitalWrite(relayPins[rFan], !fanOn);  //  change as needed "!fanOn" or "fanOn"
    if (!fanOn) setHeat(false);  // no FAN no HEAT
  }
  
  void setHeat(bool heat) {   
    heatOn = heat;
    if (!fanOn) heatOn = false;  // OFF if fan is OFF
    digitalWrite(relayPins[rHeat], !heatOn);  // change as needed "!heatOn" or "heatOn"
  }

  void doRed() {
    if (!fryerStart || fryerPause) return;
    touchDetect[tRed] = 2;
    touchDetect[tGreen] = 0;
    tDebounce.tmrStart(true);  // start debounce time
    remainingTime[tPause] = 300;  // after 5 minutes STOP
    fryerPause = true;
    setFan(false);
    webSocket.broadcastTXT( "|pause|");
  }
  
  void doGreen() {
    if (!fryerStart || !fryerPause) return;
    touchDetect[tGreen] = 2;
    touchDetect[tRed] = 0;
    tDebounce.tmrStart(true);  // start debounce time
    if (stageNr == sStage1_1) {
      stageNr = sStage2_1;
      setStage(sStage2_1);
    }
    fryerPause = false;
    setFan(true);
    webSocket.broadcastTXT("|get on|");
  }

  void doMsg() {
    uint8_t mm[2] = {0, 0},
            ss[2] = {0, 0};
    for (uint8_t i = tGo; i <= tShake; i++) {
      if (remainingTime[i] > 0) {
        mm[i] = remainingTime[i] / 60;          // seconds to minutes
        ss[i] = remainingTime[i] - mm[i] * 60;  // rest seconds
      }
    }
    sprintf(msgBuf, "|rs0|%u|%8u", heatOn, sFryer.sThermResistance);
    if (useProbe) sprintf(msgBuf + strlen(msgBuf), ", %10u", sProbe.sThermResistance);
    strcat(msgBuf, "|");
    webSocket.broadcastTXT(msgBuf);
    uint8_t n = 0;
    switch (stageNr) {
      case sStage1_1: sprintf(msgBuf, "|rs1|%3u|", temperature[nFryer]); break;
      case sStage2_1: n = 2;        // 2, 3 and 4
      case sStage3_1: if(!n) n = 3;   
      case sStage4_1: if(!n) n = 4;
                      if (useProbe) sprintf(msgBuf, "|rs3|%3u, %3u|", temperature[nFryer], temperature[nProbe]);
                      else  sprintf(msgBuf, "|rs%u|%3u, %3u:%02u, %3u:%02u|", n, temperature[nFryer], mm[tGo], ss[tGo], mm[tShake], ss[tShake]); 
                      break;
      case sStage5_2: sprintf(msgBuf, "|rs5|%8u:%02u|", mm[tGo], ss[tGo]); break;
    }
    webSocket.broadcastTXT(msgBuf);
  }

  void getTemp(uint8_t n) {
    uint32_t adc = sAdcRead(sAdcPin[nFryer]);  // default: take the average of "2" measurements
    sFryer.sThermGetTemp(adc);  // *C
    temperature[nFryer] = !fahrenheit ? sFryer.sThermTemperature : sFryer.sThermTemperature * 1.8 + 32.0;
    if (n = nProbe) {
      adc = sAdcRead(sAdcPin[nProbe]);  // default: take the average of "2" measurements
      temperature[nProbe] =  !fahrenheit ? sProbe.sThermTemperature : sProbe.sThermTemperature * 1.8 + 32.0;
    }
  }

  bool msgParse() {
    uint8_t n = 0;
    char * v;
    v = strchr(msgBuf, '|');
    while (v != NULL) {n++; v = strchr(v + 1, '|');}
    if (n >= 2) {
      uint8_t i = 0;
      v = strtok (msgBuf, "|");
      while (v != NULL) {strcpy(msgFields[i++], v); v = strtok (NULL, "|");}
    }
    else return false;
    return true;
  }

  void redInterrupt() {
    if (!touchDetect[tRed]) touchDetect[tRed] = 1;
  }
  
  void greenInterrupt() {
    if (!touchDetect[tGreen]) touchDetect[tGreen] = 1;    
  }

  void webSocketEvent(uint8_t num, WStype_t type, uint8_t * payload, size_t len) {
    switch (type) {
      case WStype_DISCONNECTED: break;
      case WStype_CONNECTED: { webSocket.sendTXT(num, "Connected"); break;}
      case WStype_TEXT: {
        if (payload[0] == 124 && payload[len - 1] == 124) {
          strcpy(msgBuf, (char*)payload);
          doCmd();
        }
        break;
      }
    }
  }

#endif
