//      www.kendziorra.nl
// (Philips) AirFryer project v2
// nov 2019, free to edit and use
/////////////////////////////////
//          CAUTION            //
//  Use this AirFryer project  //
//    only at you own risk!    //
//    I will not accept any    //
//          liability!         //
/////////////////////////////////
// use a ESP32 DEV-KIT module  //
/////////////////////////////////

// FIRST TO DO: edit "EditMe.h" and install the lib.files described in "AirFryer.h" 
#include <WiFi.h>
#include <ArduinoOTA.h>
#include "AirFryer.h"

void wifiOn(const char * ssid = "edit-edit"@,       // universal ESP WiFi (auto re)connection ...
            const char * password = "edit-edit"@);  // ...(un)comment options in function

void setup() {
  Serial.begin(115200);
  Serial.println("...\n");
  pinMode(relayPins[rFan], OUTPUT);
  pinMode(relayPins[rHeat], OUTPUT);
  doStop();
  wifiOn();
  ArduinoOTA.setHostname(myID);
  ArduinoOTA.begin();
  sFryer.sThermInit(sR[0], sT[0], ser[0], 4095);  // NTC: 104TC  -  4095: ESP32 ADC_WIDTH_BIT_12 width;
  sProbe.sThermInit(sR[1], sT[1], ser[1], 4095);  // NTC: 204TC
  touchAttachInterrupt(touchPins[tRed], redInterrupt, 40);
  touchAttachInterrupt(touchPins[tGreen], greenInterrupt, 40);
  webSocket.begin();
  webSocket.onEvent(webSocketEvent);
}

void loop() {
  wifiOn();
  ArduinoOTA.handle();
  webSocket.loop();
  if (tTimer.tmrTime() && fryerStart) doStages();
  if(touchDetect[tRed] == 1) doRed();
  else if(touchDetect[tGreen] == 1) doGreen();
  if (tDebounce.tmrTime()) {
    tDebounce.tmrStart(false);  // stop debounce time
    touchDetect[tRed] = 0;
    touchDetect[tGreen] = 0;
  }
}

void wifiOn(const char * ssid, const char * password) {
  // universal ESP WiFi (auto re)connection (un)comment options as required
  static bool isInit = false;
  static uint8_t attempt = 0;
  if (!isInit) {  // first time init?
    IPAddress local_IP(192, 168, edit, edit)@,  // if using a static IP edit the values
            gateway_IP(192, 168, edit, edit)@,
             subnet_IP(255, 255, 255,   0);
    //IPAddress primDNS(8, 8, 8, 8),  // is needed for SNTP time server ...
    //           secDNS(8, 8, 4, 4);  // ... i think all 8 values are OK
    WiFi.mode(WIFI_STA);      // choose a WiFi mode, Station or/and AccessPoint
    //WiFi.mode(WIFI_AP); 
    //WiFi.mode(WIFI_AP_STA); 
    WiFi.config(local_IP, gateway_IP, subnet_IP);  // if using a static IP
    //WiFi.config(local_IP, gateway_IP, subnet_IP, primDNS, secDNS); // static IP and SNTP time server
    WiFi.setAutoReconnect(true);
    WiFi.begin(ssid, password);
    for (uint8_t i = 0; i < 5; i++) {  // no connection: keep trying
      uint8_t n = 0;
      while (!WiFi.isConnected() && ++n < 25) delay(200);  // 0...5 sec.
      if (WiFi.isConnected()) break;
      if (i == 4) {
        delay(10000);
        ESP.restart();
      }
      WiFi.begin();
    }
    WiFi.setSleep(false);
    //if (WiFi.isConnected()) printf("WiFi connected!\n");  // (un)comment as wished
    isInit = true;
  }
  else {
    if (!WiFi.isConnected()) {
      attempt++;
      WiFi.begin();
      uint8_t n = 0;
      while (!WiFi.isConnected() && ++n < 25) delay(200);  // 0...5 sec.
      if (WiFi.isConnected()) {
        attempt = 0;
        //printf("WiFi reconnected!\n");  // (un)comment as wished
      }
      else if (attempt == 5) ESP.restart();  // (un)comment as wished
    }
  }
}
